<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Block_Adminhtml_Document_View_Status extends Mage_Adminhtml_Block_Template
{
    protected $_status = null;

    protected function _construct()
    {
        $this->setData('template', 'kiwiz/document_view_status.phtml');
        parent::_construct();
    }

    public function getKiwizStatus()
    {
        if($this->_status == null) {
            $parent = $this->getParentBlock();
            $status = null;
            if ($parent) {
                switch ($parent->getType()) {
                    case 'adminhtml/sales_order_view_tab_info':
                        $order = $parent->getOrder();
                        $collection = Mage::getResourceModel('sales/order_collection')->addFieldToFilter('entity_id', $order->getId());
                        $select = $collection->getSelect();
                        $select->join(
                            array('s' => Mage::helper('kiwiz')->getOrderStatusTableExpression()),
                            "`main_table`.`entity_id`=`s`.`order_id`",
                            array('s.kiwiz_status')
                        );
                        $select->limit(1);

                        $o = $collection->getFirstItem();
                        $status = $o->getId() ? $o->getKiwizStatus() : null;
                        break;

                    case 'adminhtml/sales_order_invoice_view_form':
                        $invoice = $parent->getInvoice();
                        $status = $invoice->getKiwizStatus();
                        break;

                    case 'adminhtml/sales_order_creditmemo_view_form':
                        $creditmemo = $parent->getCreditmemo();
                        $status = $creditmemo->getKiwizStatus();
                        break;
                }
            }
            $this->_status = $status;
        }
        return Mage::getModel('kiwiz/source_status')->getStatusLabel((int) $this->_status);
    }

    public function getKiwizMessage()
    {
        $parent = $this->getParentBlock();
        if ($parent) {
            switch ($parent->getType()) {
                case 'adminhtml/sales_order_invoice_view_form':
                    $invoice = $parent->getInvoice();
                    return $invoice->getKiwizMessage();
                    break;

                case 'adminhtml/sales_order_creditmemo_view_form':
                    $creditmemo = $parent->getCreditmemo();
                    return $creditmemo->getKiwizMessage();
                    break;
            }
        }
        return null;
    }

    public function getKiwizStatusImgUrl()
    {
        $this->getKiwizStatus();
        return Mage::helper('kiwiz')->getKiwizStatusImgUrl($this->_status);
    }

    public function getLogoUrl()
    {
        return Mage::helper('kiwiz')->getLogoUrl();
    }
}