<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Helper_Conflict extends Mage_Core_Helper_Abstract {

    protected $_moduleList;

    public function getRewrites()
    {
        //retrieve all config.xml
        $tConfigFiles = $this->getConfigFilesList();

        //parse all config.xml
        $rewrites = array();
        foreach ($tConfigFiles as $configFile) {
            $rewrites = $this->getRewriteForFile($configFile, $rewrites);
        }

        return $rewrites;
    }

    public function getConfigFilesList()
    {
        $buffer = array();
        $codePath = Mage::getBaseDir().DS.'app'.DS.'code';

        $locations = array();
        $locations[] = $codePath . '/local/';
        $locations[] = $codePath . '/community/';

        foreach ($locations as $location) {
            //parse every sub folders (means extension folders)
            $poolDir = opendir($location);
            while ($namespaceName = readdir($poolDir)) {
                if (!$this->directoryIsValid($namespaceName))
                    continue;

                //parse modules within namespace
                $namespacePath = $location . $namespaceName . '/';
                $namespaceDir = opendir($namespacePath);
                while ($moduleName = readdir($namespaceDir)) {
                    if (!$this->directoryIsValid($moduleName))
                        continue;


                    //check if it is a real active module and not backup badly nammed of a module
                    $key = $namespaceName.'_'.$moduleName;
                    if ($this->checkModulePresence($key)) {
                        $modulePath = $namespacePath . $moduleName . '/';
                        $configXmlPath = $modulePath . 'etc/config.xml';

                        if (file_exists($configXmlPath))
                            $buffer[] = $configXmlPath;
                    }
                }
                closedir($namespaceDir);
            }
            closedir($poolDir);
        }

        return $buffer;
    }

    protected function directoryIsValid($dirName)
    {
        switch ($dirName) {
            case '.':
            case '..':
            case '.DS_Store':
            case '':
                return false;
                break;
            default:
                return true;
                break;
        }
    }

    protected function manageModule($moduleName)
    {
        switch ($moduleName) {
            case 'global':
                return false;
                break;
            default:
                return true;
                break;
        }

    }

    public function getAllowedNodeTypes()
    {
        return array_keys($this->getFollows());
    }

    public function getFollows($type=null)
    {
        return array(
            'models' => array(
                'sales/models_order_invoice',
                'sales/models_order_pdf_invoice',
                'sales/models_order_creditmemo',
                'sales/models_order_pdf_creditmemo',
                'core/models_email_template_mailer',
                'adminhtml/models_config_data'
            )
        );
    }

    public function isFollowed($type, $name)
    {
        $follows = $this->getFollows();

        if(!isset($follows[$type]))
            return false;

        if(!in_array($name, $follows[$type]))
            return false;

        return true;
    }

    public function getRewriteForFile($configFilePath, $results)
    {
        try {
            //load xml
            $xmlContent = file_get_contents($configFilePath);
            $domDocument = new DOMDocument();
            $domDocument->loadXML($xmlContent);

            //parse every node types
            $nodeTypes = $this->getAllowedNodeTypes();

            foreach ($nodeTypes as $nodeType) {
                if (!$domDocument->documentElement)
                    continue;

                foreach ($domDocument->documentElement->getElementsByTagName($nodeType) as $nodeTypeMarkup) {
                    foreach ($nodeTypeMarkup->getElementsByTagName('rewrite') as $markup) {
                        //parse child nodes
                        $moduleName = $markup->parentNode->tagName;
                        if ($this->manageModule($moduleName)) {
                            foreach ($markup->getElementsByTagName('*') as $childNode) {
                                //get information
                                $className = $nodeType . '_' . $childNode->tagName;
                                $rewriteClass = $childNode->nodeValue;

                                $key = $moduleName . '/' . $className;
                                if($this->isFollowed($nodeType, $key)) {
                                    if ($this->checkModulePresence($this->getModuleKeyName($rewriteClass))) {
                                        //add to result

                                        if (!isset($results[$key]))
                                            $results[$key] = array();
                                        $results[$key][] = $rewriteClass;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (Exception $ex) {
            return $results;
        }

        return $results;
    }

    public function getModuleKeyName($rewriteClass)
    {
        $moduleKeyName = '';
        $array = explode('_',$rewriteClass);
        if(count($array)>2){
            $moduleKeyName = $array[0].'_'.$array[1];
        }
        return $moduleKeyName;
    }

    public function checkModulePresence($moduleKeyName)
    {
        $presentAndActive = false;

        if ($moduleKeyName) {
            if (!$this->_moduleList) {
                $this->_moduleList = Mage::getConfig()->getNode('modules')->children();
            }

            if (array_key_exists($moduleKeyName, $this->_moduleList)) {
                $moduleConf = $this->_moduleList->$moduleKeyName;

                //$moduleConf is a Mage_Core_Model_Config_Element
                if ($moduleConf->is("active", "true")) {
                    $presentAndActive = true;
                }
            }
        }
        return $presentAndActive;
    }

}
