<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Helper_Data extends Mage_Core_Helper_Abstract
{
    protected $_storeList = null;

    public function getOrderStatusTableExpression()
    {
        $resource = Mage::getSingleton('core/resource');
        return new Zend_Db_Expr("(
            SELECT 
                o.`entity_id` as `order_id`,
                 IF(
					GREATEST( IFNULL(`i`.`kiwiz_invoice_status`,'".MA_Kiwiz_Model_Source_Status::SENT."') , IFNULL(`c`.`kiwiz_creditmemo_status`,'".MA_Kiwiz_Model_Source_Status::SENT."') ) = '".MA_Kiwiz_Model_Source_Status::DO_NOT_SEND."',
                    '".MA_Kiwiz_Model_Source_Status::DO_NOT_SEND."',
                    LEAST( IFNULL(`i`.`kiwiz_invoice_status`,'".MA_Kiwiz_Model_Source_Status::NO_DOC."') , IFNULL(`c`.`kiwiz_creditmemo_status`,'".MA_Kiwiz_Model_Source_Status::SENT."') )
                 ) as `kiwiz_status`
            FROM `{$resource->getTableName('sales_flat_order_grid')}` AS `o`
            LEFT JOIN (
                SELECT 
                  `order_id`, 
                  IF(MAX(`kiwiz_status`)='".MA_Kiwiz_Model_Source_Status::DO_NOT_SEND."',MAX(`kiwiz_status`), MIN(`kiwiz_status`)) AS `kiwiz_invoice_status`
                FROM `{$resource->getTableName('sales_flat_invoice')}`
                GROUP BY `order_id`
            ) AS `i` ON `o`.`entity_id`=`i`.`order_id`
            LEFT JOIN (
                SELECT 
                  `order_id`, 
                  IF(MAX(`kiwiz_status`)='".MA_Kiwiz_Model_Source_Status::DO_NOT_SEND."',MAX(`kiwiz_status`), MIN(`kiwiz_status`)) AS `kiwiz_creditmemo_status`
                FROM `{$resource->getTableName('sales_flat_creditmemo')}`
                GROUP BY `order_id`
            ) AS `c` ON `o`.`entity_id`=`c`.`order_id`
        )");
    }

    public function getKiwizStatusImgUrl($status)
    {
        if($status == MA_Kiwiz_Model_Source_Status::DO_NOT_SEND) return null;
        return $this->getBooleanPictoUrl($status);
    }

    public function getBooleanPictoUrl($state)
    {
        $url = Mage::getDesign()->getSkinBaseUrl(array('_area'=>'adminhtml', '_theme'=>'default')).'images/kiwiz/';
        return $state > 0 ? $url.'checked.png' : ( $state < 0 ? $url.'warning.png' : $url.'cancel.png');
    }

    public function getLogoUrl()
    {
        return Mage::getDesign()->getSkinBaseUrl(array('_area'=>'adminhtml', '_theme'=>'default')).'images/kiwiz/logo.png';
    }

    public function getQuotas()
    {
        $responses = array();

        $stores = Mage::app()->getStores();
        foreach ($stores as $store) {
            $fullName = $store->getName();

            $group = $store->getGroup();
            $fullName = $group->getName() . ' / ' . $fullName;

            $website = $group->getWebsite();
            $fullName = $website->getName() . ' / ' . $fullName;

            try {
                $client = Mage::getModel('kiwiz/api_client', $store->getId());
                $response = $client->getQuotaInfo();
                $flag = Mage::getModel('kiwiz/flag_init');
                $flag->setFlagCodeParams($store->getId())->loadSelf();
                if(!$flag->getFlagData())
                    $flag->setFlagData(true)->save();
            } catch (Exception $e) {
                $response = new Varien_Object();
                $response->setError($this->__('Cannot get quotas'));
            }
            $response->setLabel($fullName);
            $response->setStore($store);
            $responses[] = $response;
        }

        return $responses;
    }

    public function getDocumentLabel($docCode)
    {
        switch($docCode) {
            case 'creditmemo':
                return $this->__('Credit memo');
                break;

            case 'invoice':
                return $this->__('Invoice');
                break;

            default:
                return $this->__($docCode);
                break;
        }
    }

    public function getVersion()
    {
        return (string) Mage::getConfig()->getNode('modules/MA_Kiwiz/version');
    }
}