<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Helper_Tax extends Mage_Tax_Helper_Data
{
    public function getKwConfig($confId)
    {
        $conf = Mage::getModel('kiwiz/tax_config');
        $conf->loadConfig($confId);
        return $conf;
    }


    public function getKwTaxes($source, $current=null)
    {
        $taxes = $this->getAllTaxes($source, $current);
        $kwData = array();
        foreach ($taxes as $tax) {
            $kwData[] = array(
                'tax_name' => $tax['title'] . ($tax['percent'] ? ' ('.number_format($tax['percent'], 2, '.', '').'%)' : ''),
                'tax_value' => number_format(round($tax['base_tax_amount'], 2), 4, '.', '')
            );
        }
        return $kwData;
    }

    public function getKwTaxesForItem($item)
    {
        $taxes = $this->getAllTaxesForItem($item);
        $kwData = array();
        foreach ($taxes as $tax) {
            $kwData[] = array(
                'tax_name' => $tax['title'] . ($tax['percent'] ? ' ('.number_format($tax['percent'], 2, '.', '').'%)' : ''),
                'tax_value' => number_format(round($tax['base_tax_amount'], 2), 4, '.', '')
            );
        }
        return $kwData;
    }

    public function getKwShippingTax($source, $current=null)
    {
        $taxes = $this->getShippingTax($source, $current);
        $kwData = array();
        foreach ($taxes as $tax) {
            $kwData[] = array(
                'tax_name' => $tax['title'] . ($tax['percent'] ? ' ('.number_format($tax['percent'], 2, '.', '').'%)' : ''),
                'tax_value' => number_format(round($tax['base_tax_amount'], 2), 4, '.', '')
            );
        }
        return $kwData;
    }

    public function getAllTaxes($source, $current=null)
    {
        if($current === null) $current = $source;
        $taxClassAmount = $this->getCalculatedTaxes($source, $current);
        $weeeAmount = $this->getAllWeee($current);
        $allTaxes = array_merge($taxClassAmount, $weeeAmount);
        return $allTaxes;
    }

    public function getAllTaxesForItem($item)
    {
        $taxClassAmount = $this->getCalculatedTaxForItem($item);
        $weeeAmount = $this->getAllWeeForItem($item);
        $allTaxes = array_merge($taxClassAmount, $weeeAmount);
        return $allTaxes;
    }

    public function getCalculatedTaxes($source, $current=null)
    {
        if($current === null) $current = $source;

        $taxClassAmount = array();
        if ($current && $source) {
            if ($current == $source) {
                // use the actuals
                $rates = $this->_getTaxRateSubtotals($source);
                foreach ($rates['items'] as $rate) {
                    $taxClassId = $rate['tax_id'];
                    $taxClassAmount[$taxClassId]['tax_amount'] = $rate['amount'];
                    $taxClassAmount[$taxClassId]['base_tax_amount'] = $rate['base_amount'];
                    $taxClassAmount[$taxClassId]['title'] = $rate['title'];
                    $taxClassAmount[$taxClassId]['percent'] = $rate['percent'];
                }
            } else {
                // regenerate tax subtotals
                // Calculate taxes for shipping
                $shippingTaxAmount = $current->getShippingTaxAmount();
                if ($shippingTaxAmount) {
                    $shippingTax    = $this->getShippingTax($source, $current);
                    $taxClassAmount = array_merge($taxClassAmount, $shippingTax);
                }

                foreach ($current->getItemsCollection() as $item) {
                    $taxClassAmountItem = $this->getCalculatedTaxForItem($item, false);
                    foreach($taxClassAmountItem as $taxClassId => $tax) {
                        if (isset($taxClassAmount[$taxClassId])) {
                            $taxClassAmount[$taxClassId]['tax_amount'] += $tax['tax_amount'];
                            $taxClassAmount[$taxClassId]['base_tax_amount'] += $tax['base_tax_amount'];
                        } else {
                            $taxClassAmount[$taxClassId]['tax_amount'] = $tax['tax_amount'];
                            $taxClassAmount[$taxClassId]['base_tax_amount'] = $tax['base_tax_amount'];
                            $taxClassAmount[$taxClassId]['title'] = $tax['title'];
                            $taxClassAmount[$taxClassId]['percent'] = $tax['percent'];
                        }
                    }
                }
            }

            foreach ($taxClassAmount as $key => $tax) {
                if ($tax['tax_amount'] == 0 && $tax['base_tax_amount'] == 0) {
                    unset($taxClassAmount[$key]);
                }
            }

            $taxClassAmount = array_values($taxClassAmount);
        }

        return $taxClassAmount;
    }

    public function getCalculatedTaxForItem($item, $values=true)
    {
        $taxClassAmount = array();

        $taxCollection = Mage::getResourceModel('tax/sales_order_tax_item')
            ->getTaxItemsByItemId(
                $item->getOrderItemId() ? $item->getOrderItemId() : $item->getItemId()
            );



        foreach ($taxCollection as $tax) {

            $amounts = $this->calcItemTaxAmount($item, $tax['percent'], $tax['tax_id']);

            if (isset($taxClassAmount[$tax['tax_id']])) {
                $taxClassAmount[$tax['tax_id']]['tax_amount'] += $amounts['amount'];
                $taxClassAmount[$tax['tax_id']]['base_tax_amount'] += $amounts['base_amount'];
            } else {
                $taxClassAmount[$tax['tax_id']]['tax_amount'] = $amounts['amount'];
                $taxClassAmount[$tax['tax_id']]['base_tax_amount'] = $amounts['base_amount'];
                $taxClassAmount[$tax['tax_id']]['title'] = $tax['title'];
                $taxClassAmount[$tax['tax_id']]['percent'] = $tax['percent'];
            }
        }
        return $values ? array_values($taxClassAmount) : $taxClassAmount;
    }

    protected function _defaultCalculationValues()
    {
        return array(
            'amount' => null,
            'base_amount' => null
        );
    }

    protected function _unitBaseCalculation($item, $taxPercent, $config)
    {
        $price = $item->getPrice();
        $basePrice = $item->getBasePrice();
        $qty = $item->getQty();

        if ($config->applyTaxAfterDiscount()) {
            $price = $price - $item->getDiscountAmount()/$qty + $item->getHiddenTaxAmount()/$qty;
            $basePrice = $basePrice - $item->getBaseDiscountAmount()/$qty + $item->getBaseHiddenTaxAmount()/$qty;
        }

        $taxAmounts = array(
            'amount' => $this->getCalculator()->round($qty * $this->getCalculator()->calcTaxAmount($price, $taxPercent, false, true)),
            'base_amount' => $this->getCalculator()->round($qty * $this->getCalculator()->calcTaxAmount($basePrice, $taxPercent, false, true))
        );

        return $taxAmounts;
    }

    protected function _rowBaseCalculation($item, $taxPercent, $config)
    {
        $price = $item->getRowTotal();
        $basePrice = $item->getBaseRowTotal();
        if ($config->applyTaxAfterDiscount()) {
            $price = $price - $item->getDiscountAmount() + $item->getHiddenTaxAmount();
            $basePrice = $basePrice - $item->getBaseDiscountAmount() + $item->getBaseHiddenTaxAmount();
        }

        $taxAmounts = array(
            'amount' => $this->getCalculator()->calcTaxAmount($price, $taxPercent, false, true),
            'base_amount' => $this->getCalculator()->calcTaxAmount($basePrice, $taxPercent, false, true)
        );

        return $taxAmounts;
    }

    protected function _totalBaseCalculation($item, $taxPercent, $config)
    {
        $taxAmounts = $this->_defaultCalculationValues();

        return $taxAmounts;
    }

    public function calcItemTaxAmount($item, $taxPercent, $configId)
    {
        $store = Mage::app()->getStore($item->getStoreId());

        /** @var MA_Kiwiz_Model_Tax_Config $config */
        $config = $this->getKwConfig($configId);

        switch ($config->getAlgorithm()) {
            case Mage_Tax_Model_Calculation::CALC_UNIT_BASE:
                return $this->_unitBaseCalculation($item, $taxPercent, $config);
                break;
            case Mage_Tax_Model_Calculation::CALC_ROW_BASE:
                return $this->_rowBaseCalculation($item, $taxPercent, $config);
                break;
            case Mage_Tax_Model_Calculation::CALC_TOTAL_BASE:
                return $this->_totalBaseCalculation($item, $taxPercent, $config);
                break;
            default:
                return $this->_defaultCalculationValues();
                break;
        }
    }

    public function getShippingTax($source, $current=null)
    {
        if($current === null) $current = $source;

        $taxClassAmount = array();
        if ($current && $source) {
            if ($current->getShippingTaxAmount() != 0 && $current->getBaseShippingTaxAmount() != 0) {
                $taxClassAmount[0]['tax_amount'] = $current->getShippingTaxAmount();
                $taxClassAmount[0]['base_tax_amount'] = $current->getBaseShippingTaxAmount();
                if ($current->getShippingHiddenTaxAmount() > 0) {
                    $taxClassAmount[0]['hidden_tax_amount'] = $current->getShippingHiddenTaxAmount();
                }
                $taxClassAmount[0]['title'] = 'Shipping tax';
                $taxClassAmount[0]['percent'] = null;
            }
        }

        return $taxClassAmount;
    }

    public function getAllWeee($source)
    {
        $allWeee = array();
        $store = $this->_app->getStore();

        if (!Mage::helper('weee')->includeInSubtotal($store)) {
            foreach ($source->getAllItems() as $item) {
                $allWeeeForItem = $this->getAllWeeForItem($item, false);
                foreach ($allWeeeForItem as $title => $weee) {
                    if (!array_key_exists($title, $allWeee)) {
                        $allWeee[$title] = array(
                            'tax_amount' => 0,
                            'base_tax_amount' => 0,
                            'title' => $title,
                            'percent' => null
                        );
                    }
                    $allWeee[$title]['tax_amount'] += $weee['tax_amount'];
                    $allWeee[$title]['base_tax_amount'] += $weee['base_tax_amount'];
                }
            }
        }

        return array_values($allWeee);
    }

    public function getAllWeeForItem($item, $values=true)
    {
        $allWeee = array();
        foreach (Mage::helper('weee')->getApplied($item) as $tax) {
            $weeeDiscount = isset($tax['weee_discount']) ? $tax['weee_discount'] : 0;
            $base_weeeDiscount = isset($tax['base_weee_discount']) ? $tax['base_weee_discount'] : 0;

            $title = $tax['title'];

            $rowAmount = isset($tax['row_amount']) ? $tax['row_amount'] : 0;
            $base_rowAmount = isset($tax['base_row_amount']) ? $tax['base_row_amount'] : 0;

            $rowAmountInclTax = isset($tax['row_amount_incl_tax']) ? $tax['row_amount_incl_tax'] : 0;
            $base_rowAmountInclTax = isset($tax['base_row_amount_incl_tax']) ? $tax['base_row_amount_incl_tax'] : 0;

            if (!array_key_exists($title, $allWeee)) {
                $allWeee[$title] = array(
                    'tax_amount' => 0,
                    'base_tax_amount' => 0,
                    'title' => $title,
                    'percent' => null
                );
            }
            $allWeee[$title]['tax_amount'] += $rowAmount - $weeeDiscount;
            $allWeee[$title]['base_tax_amount'] += $base_rowAmount - $base_weeeDiscount;
        }
        return $values ? array_values($allWeee) : $allWeee;
    }
}