<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Model_Cron_Quotas_Check
{
    protected $_limits = null;
    protected $_notificationEmails = null;
    protected $_allLimits = null;
    protected $_flag = null;

    protected function _getLimits()
    {
        if($this->_limits === null) {
            $this->_limits = Mage::helper('kiwiz/config')->getQuotaPercentAlert();
        }
        return $this->_limits;
    }

    protected function _getNotificationEmails()
    {
        if($this->_notificationEmails === null) {
            $this->_notificationEmails = Mage::helper('kiwiz/config')->getNotificationEmails();
        }
        return $this->_notificationEmails;
    }

    protected function _getAllLimits()
    {
        if($this->_allLimits === null) {
            $this->_allLimits = Mage::getModel('kiwiz/source_quotas_percent')->toArray();
        }
        return $this->_allLimits;
    }

    protected function _getFlag($storeId, $percentLimit)
    {
        $flag = Mage::getModel('kiwiz/flag_quotas_notification');
        $flag->setFlagCodeParams($storeId, $percentLimit)->loadSelf();
        return (bool) $flag->getFlagData();
    }

    protected function _setFlag($storeId, $percentLimit, $value)
    {
        $flag = Mage::getModel('kiwiz/flag_quotas_notification');
        $flag->setFlagCodeParams($storeId, $percentLimit)->loadSelf();
        $flag->setFlagData((bool) $value )->save();
        return $this;
    }

    protected function _calcPercents(Varien_Object $quota)
    {
        if(!$quota->hasPercent()) {
            $percent = 100;
            if($quota->getLimit()) {
                $percent = round($quota->getUsed()*100/$quota->getLimit());
            }
            $quota->setPercent($percent);

        }

        if(!$quota->hasPercentLimit()) {
            $percentLimit = 0;

            $limitInf = 0;
            foreach ($this->_getAllLimits() as $limitSup) {

                if( ($quota->getPercent() >= $limitInf) && ($quota->getPercent() < $limitSup) )
                    $percentLimit = $limitInf;
                elseif ($quota->getPercent() == $limitSup)
                    $percentLimit = $limitSup;

                $limitInf = $limitSup;
            }

            $quota->setPercentLimit($percentLimit);
        }
    }

    public function run()
    {
        if( !count($this->_getNotificationEmails()) || !count($this->_getLimits()))
            return;

        $h = Mage::helper('kiwiz');
        $quotas = $h->getQuotas();
        $quotasAlerts = array();
        $message = array();
        foreach ($quotas as $quota) {
            $this->_calcPercents($quota);
            foreach ($this->_getAllLimits() as $limit) {
                if($limit != $quota->getPercentLimit()) {
                    $this->_setFlag($quota->getStore()->getId(), $limit, false);
                } else {
                    if(!in_array($limit, $this->_getLimits())) {
                        $this->_setFlag($quota->getStore()->getId(), $limit, false);
                    } else {
                        if (!$this->_getFlag($quota->getStore()->getId(), $limit)) {
                            $quotasAlerts[] = $quota;
                            $message[] = $h->__('Usage for Magento store "%s" exceeded %s', $quota->getLabel(), $quota->getPercentLimit().'%');
                        }
                    }
                }
            }
        }

        if(!count($quotasAlerts))
            return;

        if(Mage::helper('kiwiz/mail')->adminNotify(
            $this->_getNotificationEmails(),
            $h->__('Alert on Kiwiz quotas'),
            nl2br(implode(PHP_EOL, $message))
        )) {
            foreach ($quotasAlerts as $quota) {
                $this->_setFlag($quota->getStore()->getId(), $quota->getPercentLimit(), true);
            }
        }
    }
}