<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Model_Document_Invoice extends MA_Kiwiz_Model_Document_Abstract
{
    protected function _getDocumentName()
    {
        return 'invoice';
    }

    protected function _getObjectData($object)
    {
        $storeId = $object->getStore()->getId();
        $order = $object->getOrder();
        $shipping = $order->getShippingAddress();
        $billing = $order->getBillingAddress();

        $items = $object->getAllItems();

        $products = array();
        $parentItems = array();
        foreach($items as $item) {
            $orderItem = $item->getOrderItem();
            $parentItem = $orderItem->getParentItem();
            $parentItems[$item->getId()] = array(
                'order_item' => $orderItem,
                'parent_item' => $parentItem
            );

            if ($parentItem)
                $products[$parentItem->getId()] = $item->getProductId();
        }

        $itemsData = array();

        foreach($items as $item) {

            if($parentItems[$item->getId()]['parent_item']) continue;

            $productId = isset($products[$parentItems[$item->getId()]['order_item']->getId()]) ? $products[$parentItems[$item->getId()]['order_item']->getId()] : $item->getProductId();
            $manufacturer = Mage::getResourceModel('catalog/product')->getAttributeRawValue(
                $productId,
                $this->_getManufacturerAttributeCode($storeId),
                $storeId
            );
            $ean13 = Mage::getResourceModel('catalog/product')->getAttributeRawValue(
                $productId,
                $this->_getEan13AttributeCode($storeId),
                $storeId
            );

            $itemsData[] = array(
                'sku' => $item->getSku(),
                'ean13' => $ean13,
                'product_name' => $item->getName(),
                'manufacturer' => $manufacturer,
                'qty' => number_format($item->getQty(), 4, '.', ''),
                'row_total_excl_tax' => number_format(
                    $item->getRowTotalInclTax()-$item->getTaxAmount() > 0 ? $item->getRowTotalInclTax()-$item->getTaxAmount() : 0,
                    4,
                    '.',
                    ''
                ),
                'row_total_tax_amount' => Mage::helper('kiwiz/tax')->getKwTaxesForItem($item)
            );
        }

        $data = array(
            'increment_id'              => $object->getIncrementId(),
            'date'                      => $object->getCreatedAt(),
            'email'                     => $order->getCustomerEmail(),
            'billing_address'           => array(
                'firstname'     => $billing->getFirstname(),
                'lastname'      => $billing->getLastname(),
                'company'       => $billing->getCompany(),
                'street'        => $billing->getStreetFull(),
                'postcode'      => $billing->getPostcode(),
                'city'          => $billing->getCity(),
                'country_code'  => $billing->getCountryId()
            ),
            'payment_method'            => $order->getPayment()->getMethodInstance()->getTitle(),
            'shipping_address'          => array(
                'firstname'     => $shipping->getFirstname(),
                'lastname'      => $shipping->getLastname(),
                'company'       => $shipping->getCompany(),
                'street'        => $shipping->getStreetFull(),
                'postcode'      => $shipping->getPostcode(),
                'city'          => $shipping->getCity(),
                'country_code'  => $shipping->getCountryId()
            ),
            'shipping_method'           => $order->getShippingDescription(),
            'shipping_amount_excl_tax'  => number_format(
                $object->getBaseShippingInclTax()-$order->getBaseShippingTaxAmount() > 0 ? $object->getBaseShippingInclTax()-$order->getBaseShippingTaxAmount() : 0,
                4,
                '.',
                ''
            ),
            'shipping_tax_amount'       => Mage::helper('kiwiz/tax')->getKwShippingTax($order, $object),
            'items'                     => $itemsData,
            'grand_total_excl_tax'      => number_format(
                $object->getBaseGrandTotal()-$object->getBaseTaxAmount() > 0 ? $object->getBaseGrandTotal()-$object->getBaseTaxAmount() : 0,
                4,
                '.',
                ''
            ),
            'grand_total_tax_amount'    => Mage::helper('kiwiz/tax')->getKwTaxes($order, $object)
        );

        return $data;
    }
}