<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

abstract class MA_Kiwiz_Model_Observer_Document_Abstract
{
    abstract protected function _getDocumentName();

    public function saveBefore(Varien_Event_Observer $observer)
    {
        $documentName = $this->_getDocumentName();
        $object = $observer->getEvent()->getData($documentName);
        $flag = Mage::getModel('kiwiz/flag_init');
        $flag->setFlagCodeParams($object->getStoreId())->loadSelf();
        if(!$flag->getFlagData() && $object->getData('kiwiz_status')==MA_Kiwiz_Model_Source_Status::NOT_SENT)
            $object->setData('kiwiz_status', MA_Kiwiz_Model_Source_Status::DO_NOT_SEND);
    }

    public function saveAfter(Varien_Event_Observer $observer)
    {
        $documentName = $this->_getDocumentName();

        $object = $observer->getEvent()->getData($documentName);
        if($object->getData('kiwiz_status') == MA_Kiwiz_Model_Source_Status::NOT_SENT) {
            $document = Mage::getModel('kiwiz/document_'.$documentName);
            try { $document->save($object); }
            catch ( Exception $e ) {}
        }
    }

    public function controllerActionLayoutRenderBefore(Varien_Event_Observer $observer)
    {
        $block = Mage::app()->getLayout()->getBlock('sales_'.$this->_getDocumentName().'_view');
        if($block) {
            $objectId = Mage::app()->getRequest()->getParam($this->_getDocumentName().'_id', null);
            $object = Mage::getModel('sales/order_'.$this->_getDocumentName())->load($objectId);

            if($object->getId() && $object->getKiwizStatus()==MA_Kiwiz_Model_Source_Status::NOT_SENT) {
                $url = Mage::helper("adminhtml")->getUrl('kiwiz_b/'.$this->_getDocumentName().'/send', array('id'=>$objectId));
                $block->addButton('kiwiz_send', array(
                    'label'     => Mage::helper('kiwiz')->__('Send to Kiwiz'),
                    'onclick'   => 'setLocation(\'' . $url . '\')',
                    'class'     => 'go'
                ));
            }
        }
    }

    public function printDocument(Varien_Event_Observer $observer)
    {
        $order = Mage::registry('current_order');
        $document = Mage::registry('current_'.$this->_getDocumentName());

        if($document) {
            $documents = $document;
        } else {
            $documents = Mage::getResourceModel('sales/order_'.$this->_getDocumentName().'_collection')
                ->addfieldToFilter('order_id', $order->getId());
        }

        try{
            $pdf = Mage::getModel('kiwiz/document_'.$this->_getDocumentName())->getPdf($documents);
            header("Content-type:application/pdf");
            echo $pdf->render();
            die();
        } catch (Exception $e){
            Mage::getSingleton('core/session')->addError(Mage::helper('kiwiz')->__('An error occurred, please try later'));
            $url = Mage::helper('core/http')->getHttpReferer() ?: Mage::getUrl();
            Mage::app()->getResponse()->setRedirect($url) ->sendResponse();
        }
    }
}