<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

// Compatibility with magento 1.8
class MA_Kiwiz_Model_Rest_Client extends Zend_Service_Abstract
{
    protected $_data = array();
    protected $_uri = null;
    protected $_noReset = false;

    protected $_testMode = false;
    protected $_platform = null;
    protected $_version = null;

    public function __construct($uri = null, $testMode = false, $platform=null, $version=null)
    {
        if (!empty($uri)) {
            $this->setUri($uri);
        }
        $this->setPlatform($platform);
        $this->setTestMode($testMode);
        $this->setVersion($version);
    }

    public function isTestMode()
    {
        return $this->_testMode;
    }

    public function setTestMode($testMode)
    {
        $this->_testMode = (bool) $testMode;
    }

    public function getPlatform()
    {
        return $this->_platform;
    }

    public function setPlatform($platform)
    {
        $this->_platform = $platform;
    }

    public function getVersion()
    {
        return $this->_version;
    }

    public function setVersion($version)
    {
        $this->_version = $version;
    }

    protected function _getDefaultGetParameters()
    {
        $params = array();

        if($this->isTestMode())
            $params['test_mode'] = 1;

        if($this->getPlatform())
            $params['platform'] = $this->getPlatform();

        if($this->getVersion())
            $params['version'] = $this->getVersion();

        return $params;
    }

    public function setUri($uri)
    {
        if ($uri instanceof Zend_Uri_Http) {
            $this->_uri = $uri;
        } else {
            $this->_uri = Zend_Uri::factory($uri);
        }

        return $this;
    }

    public function getUri()
    {
        return $this->_uri;
    }

    protected function _prepareRest($path)
    {
        // Get the URI object and configure it
        if (!$this->_uri instanceof Zend_Uri_Http) {
            #require_once 'Zend/Rest/Client/Exception.php';
            throw new Zend_Rest_Client_Exception('URI object must be set before performing call');
        }

        $uri = $this->_uri->getUri();

        if ($path[0] != '/' && $uri[strlen($uri)-1] != '/') {
            $path = '/' . $path;
        }

        $this->_uri->setPath($path);

        if ($this->_noReset) {
            $this->_noReset = false;
        } else {
            self::getHttpClient()->resetParameters();
        }

        ini_set('max_execution_time', 0);
        self::getHttpClient()->setUri($this->_uri);
        self::getHttpClient()->setConfig(array(
            'maxredirects' => 0,
            'timeout'      => 120
        ));
    }

    public function setNoReset($bool = true)
    {
        $this->_noReset = $bool;
    }

    public function restGet($path, array $query = null)
    {
        $this->_prepareRest($path);

        $client = self::getHttpClient();
        $client->setParameterGet($this->_getDefaultGetParameters());

        $client->setParameterGet($query);
        return $client->request('GET');
    }

    protected function _performPost($method, $data = null)
    {
        $client = self::getHttpClient();
        $client->setParameterGet($this->_getDefaultGetParameters());

        if (is_string($data)) {
            $client->setRawData($data);
        } elseif (is_array($data) || is_object($data)) {
            $client->setParameterPost((array) $data);
        }
        return $client->request($method);
    }

    public function restPost($path, $data = null)
    {
        $this->_prepareRest($path);
        return $this->_performPost('POST', $data);
    }

    public function restPut($path, $data = null)
    {
        $this->_prepareRest($path);
        return $this->_performPost('PUT', $data);
    }

    public function restDelete($path, $data = null)
    {
        $this->_prepareRest($path);
        return $this->_performPost('DELETE', $data);
    }

    public function __call($method, $args)
    {
        $methods = array('post', 'get', 'delete', 'put');

        if (in_array(strtolower($method), $methods)) {
            if (!isset($args[0])) {
                $args[0] = $this->_uri->getPath();
            }
            $this->_data['rest'] = 1;
            $data = array_slice($args, 1) + $this->_data;
            $response = $this->{'rest' . $method}($args[0], $data);
            $this->_data = array();//Initializes for next Rest method.
            return new Zend_Rest_Client_Result($response->getBody());
        } else {
            // More than one arg means it's definitely a Zend_Rest_Server
            if (sizeof($args) == 1) {
                // Uses first called function name as method name
                if (!isset($this->_data['method'])) {
                    $this->_data['method'] = $method;
                    $this->_data['arg1']  = $args[0];
                }
                $this->_data[$method]  = $args[0];
            } else {
                $this->_data['method'] = $method;
                if (sizeof($args) > 0) {
                    foreach ($args as $key => $arg) {
                        $key = 'arg' . $key;
                        $this->_data[$key] = $arg;
                    }
                }
            }
            return $this;
        }
    }
}
